require 'octopi'

module Pubba
  module Assets
    class Handler
      class << self
        include Octopi

        def resources
          @resources ||= []
        end

        def asset(file)
          raise NotImplementedError
        end

        # @return path, ext Path and extension of resource
        def get_external_resource(url)
          path  = ''
          ext   = ''

          authenticated do
            parts = url.split(':')
            user  = parts[1]
            repo  = parts[2]
            tag   = parts[3]
            file  = parts[4]

            # file extension: .js
            ext  = File.extname(file)

            # file name without extension
            basename = File.basename(file, ext)

            # subdir gitub/primedia
            subdir = File.join(parts[0], user)

            # github/repo/file
            path = File.join(subdir, basename)

            unless resources.include?(url)
              folder = (ext == '.css' ? Pubba.style_folder : Pubba.script_folder)

              # assets/js/github/repo/file
              out_path = File.join(Pubba.asset_folder, folder, subdir)
              FileUtils.mkdir_p(out_path)

              # assets/js/github/repo/file
              out_file = File.join(out_path, basename + ext)

              r = Repository.find(user: user, repo: repo)
              r.tags.each do |t|
                if t.name == tag
                  sha = t.sha
                  begin
                    blob = Blob.find(user: user, repo: repo, sha: sha, path: file)
                    File.open(out_file, "w"){|f| f.write(blob.data) }
                    break
                  rescue Octopi::APIError => e
                    puts "Error attempting to get: user: #{user}, repo: #{repo}, sha: #{sha}, path: #{file}"
                    puts e.message
                  end
                end
              end

              resources << url
            end # resources include
          end

          return path, ext.split('.').last
        end
      end # class block

      def save_as(file)
        raise NotImplementedError
      end

      def process(pattern, destination)
        raise NotImplementedError
      end

      def build(name, type, ext, urls)
        raise NotImplementedError
      end
    end # Handler

  end # Assets
end # Pubba
